/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.ooxml.docx.tools;

import java.text.ParseException;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;

import org.docx4j.IndexedNode;
import org.docx4j.IndexedNodeList;
import org.docx4j.dml.CTBlip;
import org.docx4j.dml.CTBlipFillProperties;
import org.docx4j.dml.CTGroupShapeProperties;
import org.docx4j.dml.CTGroupTransform2D;
import org.docx4j.dml.CTNonVisualDrawingProps;
import org.docx4j.dml.CTNonVisualGroupDrawingShapeProps;
import org.docx4j.dml.CTPoint2D;
import org.docx4j.dml.CTPositiveSize2D;
import org.docx4j.dml.CTTextBodyProperties;
import org.docx4j.dml.CTTextShapeAutofit;
import org.docx4j.dml.Graphic;
import org.docx4j.dml.GraphicData;
import org.docx4j.dml.picture.CTPictureNonVisual;
import org.docx4j.dml.picture.Pic;
import org.docx4j.dml.wordprocessingDrawing.Anchor;
import org.docx4j.dml.wordprocessingDrawing.CTPosH;
import org.docx4j.dml.wordprocessingDrawing.CTPosV;
import org.docx4j.dml.wordprocessingDrawing.CTWrapNone;
import org.docx4j.dml.wordprocessingDrawing.CTWrapSquare;
import org.docx4j.dml.wordprocessingDrawing.CTWrapThrough;
import org.docx4j.dml.wordprocessingDrawing.CTWrapTight;
import org.docx4j.dml.wordprocessingDrawing.CTWrapTopBottom;
import org.docx4j.dml.wordprocessingDrawing.GraphicBase;
import org.docx4j.dml.wordprocessingDrawing.Inline;
import org.docx4j.dml.wordprocessingDrawing.ObjectFactory;
import org.docx4j.dml.wordprocessingDrawing.STAlignH;
import org.docx4j.dml.wordprocessingDrawing.STAlignV;
import org.docx4j.dml.wordprocessingDrawing.STRelFromH;
import org.docx4j.dml.wordprocessingDrawing.STRelFromV;
import org.docx4j.dml.wordprocessingDrawing.STWrapText;
import org.docx4j.dml.wordprocessingGroup2010.CTWordprocessingGroup;
import org.docx4j.dml.wordprocessingShape2010.CTTextboxInfo;
import org.docx4j.dml.wordprocessingShape2010.CTWordprocessingShape;
import org.docx4j.jaxb.Context;
import org.docx4j.mce.AlternateContent;
import org.docx4j.openpackaging.exceptions.InvalidFormatException;
import org.docx4j.openpackaging.exceptions.PartUnrecognisedException;
import org.docx4j.relationships.Relationship;
import org.docx4j.wml.CTTxbxContent;
import org.docx4j.wml.Drawing;
import org.json.JSONException;
import org.json.JSONObject;

import com.openexchange.office.FilterException;
import com.openexchange.office.ooxml.docx.OperationDocument;
import com.openexchange.office.ooxml.docx.operations.CreateOperationHelper;
import com.openexchange.office.ooxml.docx.tools.Component.IShape;
import com.openexchange.office.ooxml.docx.tools.Component.ShapeType;
import com.openexchange.office.ooxml.docx.tools.Component.Type;
import com.openexchange.office.ooxml.drawingml.DMLHelper;
import com.openexchange.office.ooxml.tools.Commons;

public abstract class DrawingML {

    final static String alternateContentNS =
            "xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\" " +
            "xmlns:mc=\"http://schemas.openxmlformats.org/markup-compatibility/2006\" " +
            "xmlns:o=\"urn:schemas-microsoft-com:office:office\" " +
            "xmlns:v=\"urn:schemas-microsoft-com:vml\" " +
            "xmlns:w=\"http://schemas.openxmlformats.org/wordprocessingml/2006/main\" " +
            "xmlns:wp=\"http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing\" ";

    final static String defaultSpPr =
            "<wps:spPr>" +
            "    <a:xfrm>" +
            "        <a:off x=\"0\" y=\"0\" />" +
            "        <a:ext cx=\"100000\" cy=\"100000\" />" +
            "    </a:xfrm>" +
            "    <a:prstGeom prst=\"rect\">" +
            "        <a:avLst />" +
            "    </a:prstGeom>" +
            "    <a:noFill />" +
            "    <a:ln>" +
            "        <a:noFill />" +
            "    </a:ln>" +
            "</wps:spPr>";

    final static String defaultSpStyle =
            "<wps:style>" +
            "    <a:lnRef idx=\"2\">" +
            "        <a:schemeClr val=\"dk1\" />" +
            "    </a:lnRef>" +
            "    <a:fillRef idx=\"1\">" +
            "        <a:schemeClr val=\"lt1\" />" +
            "    </a:fillRef>" +
            "    <a:effectRef idx=\"0\">" +
            "        <a:schemeClr val=\"accent1\" />" +
            "    </a:effectRef>" +
            "    <a:fontRef idx=\"minor\">" +
            "        <a:schemeClr val=\"dk1\" />" +
            "    </a:fontRef>" +
            "</wps:style>";

    // creates an AlternateContent textframe (drawingML + VML), this shape is empty and has no textframe
    public static AlternateContent createAlternateContentShape()
        throws JAXBException {

        final String ml =
            "<mc:AlternateContent " +
            alternateContentNS +
            "xmlns:wps=\"http://schemas.microsoft.com/office/word/2010/wordprocessingShape\" " +
            ">" +
            "   <mc:Choice Requires=\"wps\">" +
            "       <w:drawing>" +
            "           <wp:inline distT=\"0\" distB=\"0\" distL=\"0\" distR=\"0\">" +
            "               <wp:extent cx=\"100000\" cy=\"100000\" />" +
            "               <wp:effectExtent l=\"0\" t=\"0\" r=\"19050\" b=\"16510\" />" +
            "               <wp:docPr id=\"217\" name=\"Textfeld 1\" />" +
            "               <wp:cNvGraphicFramePr>" +
            "               <a:graphicFrameLocks" +
            "                   xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\" />" +
            "               </wp:cNvGraphicFramePr>" +
            "               <a:graphic xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\">" +
            "                   <a:graphicData uri=\"http://schemas.microsoft.com/office/word/2010/wordprocessingShape\">" +
            "                       <wps:wsp>" +
            "                           <wps:cNvSpPr txBox=\"1\">" +
            "                               <a:spLocks noChangeArrowheads=\"1\"/>" +
            "                           </wps:cNvSpPr>" +
            defaultSpPr +
            defaultSpStyle +
            "                           <wps:bodyPr rot=\"0\" vert=\"horz\" wrap=\"square\" lIns=\"91440\" tIns=\"45720\" rIns=\"91440\" bIns=\"45720\" anchor=\"t\" anchorCtr=\"0\">" +
            "                               <a:noAutoFit/>" +
            "                           </wps:bodyPr>" +
            "                       </wps:wsp>" +
            "                   </a:graphicData>" +
            "               </a:graphic>" +
            "           </wp:inline>" +
            "       </w:drawing>" +
            "   </mc:Choice>" +
            "   <mc:Fallback>" +
            "       <w:pict>" +
            "           <v:shapetype id=\"_x0000_t202\" coordsize=\"21600,21600\" o:spt=\"202\" path=\"m,l,21600r21600,l21600,xe\">" +
            "               <v:stroke joinstyle=\"miter\"/>" +
            "               <v:path gradientshapeok=\"t\" o:connecttype=\"rect\"/>" +
            "           </v:shapetype>" +
            "           <v:shape id=\"Textfeld 1\" o:spid=\"_x0000_s1026\" type=\"#_x0000_t202\" style=\"width:185.9pt;height:110.6pt;visibility:visible;mso-wrap-style:square;mso-left-percent:-10001;mso-top-percent:-10001;mso-position-horizontal:absolute;mso-position-horizontal-relative:char;mso-position-vertical:absolute;mso-position-vertical-relative:line;mso-left-percent:-10001;mso-top-percent:-10001;v-text-anchor:top\"/>" +
            "       </w:pict>" +
            "   </mc:Fallback>" +
            "</mc:AlternateContent>";

        final java.util.HashMap<String, String> mappings = new java.util.HashMap<String, String>();
        return (AlternateContent)org.docx4j.XmlUtils.unmarshallFromTemplate(ml, mappings);
    }

    public static AlternateContent createAlternateContentGroup()
        throws JAXBException {

        final String ml =
            "<mc:AlternateContent " +
            alternateContentNS +
            "xmlns:w10=\"urn:schemas-microsoft-com:office:word\" " +
            "xmlns:wpg=\"http://schemas.microsoft.com/office/word/2010/wordprocessingGroup\" " +
            ">" +
            "    <mc:Choice Requires=\"wpg\">" +
            "        <w:drawing>" +
            "            <wp:inline distT=\"0\" distB=\"0\" distL=\"0\" distR=\"0\">" +
            "                <wp:extent cx=\"100000\" cy=\"100000\"/>" +
            "                <wp:effectExtent l=\"0\" t=\"0\" r=\"19050\" b=\"16510\"/>" +
            "                <wp:docPr id=\"1\" name=\"Group 1\"/>" +
            "                <wp:cNvGraphicFramePr/>" +
            "                <a:graphic xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\">" +
            "                    <a:graphicData uri=\"http://schemas.microsoft.com/office/word/2010/wordprocessingGroup\">" +
            "                        <wpg:wgp>" +
            "                            <wpg:cNvGrpSpPr/>" +
            "                            <wpg:grpSpPr>" +
            "                                <a:xfrm>" +
            "                                    <a:off x=\"0\" y=\"0\"/>" +
            "                                    <a:ext cx=\"100000\" cy=\"100000\"/>" +
            "                                    <a:chOff x=\"0\" y=\"0\"/>" +
            "                                    <a:chExt cx=\"19050\" cy=\"16510\"/>" +
            "                                </a:xfrm>" +
            "                            </wpg:grpSpPr>" +
            "                        </wpg:wgp>" +
            "                    </a:graphicData>" +
            "                </a:graphic>" +
            "            </wp:inline>" +
            "        </w:drawing>" +
            "    </mc:Choice>" +
            "    <mc:Fallback>" +
            "        <w:pict>" +
            "            <v:group id=\"Group 1\" o:spid=\"_x0000_s1026\" style=\"width:35.25pt;height:39pt;mso-position-horizontal-relative:char;mso-position-vertical-relative:line\">" +
            "                <w10:anchorlock/>" +
            "            </v:group>" +
            "        </w:pict>" +
            "    </mc:Fallback>" +
            "</mc:AlternateContent>";

        final java.util.HashMap<String, String> mappings = new java.util.HashMap<String, String>();
        return (AlternateContent)org.docx4j.XmlUtils.unmarshallFromTemplate(ml, mappings);
    }

    public static CTWordprocessingGroup createDrawingMLGroup() {

        final org.docx4j.dml.wordprocessingGroup2010.ObjectFactory objectFactory = new org.docx4j.dml.wordprocessingGroup2010.ObjectFactory();
        final CTWordprocessingGroup group = objectFactory.createCTWordprocessingGroup();
        final CTNonVisualDrawingProps cNvPr = Context.getDmlObjectFactory().createCTNonVisualDrawingProps();
        cNvPr.setName("Group 1");
        cNvPr.setId(1);
        group.setCNvPr(cNvPr);
        final CTNonVisualGroupDrawingShapeProps cNvGrpPr = Context.getDmlObjectFactory().createCTNonVisualGroupDrawingShapeProps();
        group.setCNvGrpSpPr(cNvGrpPr);
        final CTGroupShapeProperties grpSpPr = Context.getDmlObjectFactory().createCTGroupShapeProperties();
        group.setGrpSpPr(grpSpPr);
        final CTGroupTransform2D grpXFrm = Context.getDmlObjectFactory().createCTGroupTransform2D();
        grpSpPr.setXfrm(grpXFrm);
        return group;
    }

    public static CTWordprocessingShape createDrawingMLShape()
        throws JAXBException {

        final String ml =
            "<wps:wsp " +
            alternateContentNS +
            "xmlns:wps=\"http://schemas.microsoft.com/office/word/2010/wordprocessingShape\" " +
            ">" +
            "    <wps:cNvPr id=\"1\" name=\"Rechteck 1\" />" +
            "    <wps:cNvSpPr />" +
            defaultSpPr +
            defaultSpStyle +
            "    <wps:bodyPr rot=\"0\" spcFirstLastPara=\"0\"" +
            "        vertOverflow=\"overflow\" horzOverflow=\"overflow\" vert=\"horz\"" +
            "        wrap=\"square\" lIns=\"91440\" tIns=\"45720\" rIns=\"91440\" bIns=\"45720\"" +
            "        numCol=\"1\" spcCol=\"0\" rtlCol=\"0\" fromWordArt=\"0\" anchor=\"ctr\"" +
            "        anchorCtr=\"0\" forceAA=\"0\" compatLnSpc=\"1\">" +
            "        <a:prstTxWarp prst=\"textNoShape\">" +
            "            <a:avLst />" +
            "        </a:prstTxWarp>" +
            "        <a:noAutofit />" +
            "    </wps:bodyPr>" +
            "</wps:wsp>";

        final java.util.HashMap<String, String> mappings = new java.util.HashMap<String, String>();
        return (CTWordprocessingShape)((JAXBElement<?>)org.docx4j.XmlUtils.unmarshallFromTemplate(ml, mappings)).getValue();
    }

    public static Pic createDrawingMLPic()
        throws JAXBException {

        final String ml =
            "<pic:pic " +
            alternateContentNS +
            "xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\" " +
            "xmlns:pic=\"http://schemas.openxmlformats.org/drawingml/2006/picture\">" +
            "    <pic:nvPicPr>" +
            "        <pic:cNvPr id=\"1\" name=\"Grafik 1\" />" +
            "        <pic:cNvPicPr>" +
            "            <a:picLocks noChangeAspect=\"1\" />" +
            "        </pic:cNvPicPr>" +
            "    </pic:nvPicPr>" +
            "    <pic:blipFill>" +
            "        <a:blip r:embed=\"\" cstate=\"print\">" +
            "            <a:extLst>" +
            "                <a:ext uri=\"{28A0092B-C50C-407E-A947-70E740481C1C}\" />" +
            "            </a:extLst>" +
            "        </a:blip>" +
            "        <a:stretch>" +
            "            <a:fillRect />" +
            "        </a:stretch>" +
            "    </pic:blipFill>" +
            "    <pic:spPr>" +
            "        <a:xfrm>" +
            "            <a:off x=\"0\" y=\"0\" />" +
            "            <a:ext cx=\"1\" cy=\"1\" />" +
            "        </a:xfrm>" +
            "        <a:prstGeom prst=\"rect\">" +
            "            <a:avLst />" +
            "        </a:prstGeom>" +
            "    </pic:spPr>" +
            "</pic:pic>";

        final java.util.HashMap<String, String> mappings = new java.util.HashMap<String, String>();
        return (Pic)((JAXBElement<?>)org.docx4j.XmlUtils.unmarshallFromTemplate(ml, mappings)).getValue();
    }

    protected static abstract class DrawingML_base implements IShape {

        final CTWordprocessingGroup parentGroup;

        protected DrawingML_base(CTWordprocessingGroup parent) {
            parentGroup = parent;
        }
        protected IShape getNext(Object o) {
            if(parentGroup!=null) {
                final IndexedNodeList<Object> parentGroupContent = parentGroup.getContent();
                final int index = parentGroupContent.indexOf(o)+1;
                if(index>0&&index<parentGroupContent.size()) {
                    final Object child = parentGroupContent.get(index);
                    if(child instanceof CTWordprocessingGroup) {
                        return new DrawingML_group((CTWordprocessingGroup)child, parentGroup);
                    }
                    else if (child instanceof CTWordprocessingShape) {
                        return new DrawingML_shape((CTWordprocessingShape)child, parentGroup);
                    }
                    else if (child instanceof Pic) {
                        return new DrawingML_pic((Pic)child, parentGroup);
                    }
                }
            }
            return null;
        }
        public Object getChild() {
            return null;
        }
        public Object insertChild(int number, Component.Type type)
            throws JAXBException {

            return null;
        }
		public static void postFixNonConformance(CTNonVisualDrawingProps drawingProps, AtomicInteger id) {
			if(drawingProps!=null) {
				drawingProps.setId(id.incrementAndGet());
			}
		}
    }

    public static class DrawingML_shape extends DrawingML_base {

    	final private CTWordprocessingShape shape;

		protected DrawingML_shape(CTWordprocessingShape o, CTWordprocessingGroup parent) {
			super(parent);
			shape = o;
		}
		@Override
		public ShapeType getType() {
			return ShapeType.SHAPE;
		}
		@Override
		public IShape getNext() {
			return getNext(shape);
		}
		@Override
		public Object getChild() {
			final CTTextboxInfo txBox = shape.getTxbx();
			if(txBox==null) {
				return null;
			}
			final CTTxbxContent txBoxContent = txBox.getTxbxContent();
			if(txBoxContent==null) {
				return null;
			}
			final Component.RootComponent rootComponent = new Component.RootComponent(txBoxContent);
			return rootComponent.getNextChildComponent(null, null);
		}
		@Override
    	public Object insertChild(int number, Component.Type type) {
			CTTextboxInfo txBox = shape.getTxbx();
			if(txBox==null) {
				final org.docx4j.dml.wordprocessingShape2010.ObjectFactory wordProcessingShape2010Factory = new org.docx4j.dml.wordprocessingShape2010.ObjectFactory();
				txBox = wordProcessingShape2010Factory.createCTTextboxInfo();
				shape.setTxbx(txBox);
			}
			CTTxbxContent txBoxContent = txBox.getTxbxContent();
			if(txBoxContent==null) {
				txBoxContent = Context.getWmlObjectFactory().createCTTxbxContent();
				txBoxContent.setParent(txBox);
				txBox.setTxbxContent(txBoxContent);
			}
			final Component.RootComponent rootComponent = new Component.RootComponent(txBoxContent);
			Component c = rootComponent.getNextChildComponent(null, null);
        	if(c!=null) {
        		c = c.getComponent(number);
        	}
        	return Component.insertChildComponent(null, new IndexedNode<Object>(txBoxContent), number, c, type);
		}
		public void postFixNonConformance(AtomicInteger id) {
			postFixNonConformance(shape.getCNvPr(), id);
		}
		@Override
    	public void applyAttrsFromJSON(OperationDocument operationDocument, JSONObject attrs)
    	    throws JSONException {

		    shape.setSpPr(DMLHelper.applyShapePropertiesFromJson(shape.getSpPr(), attrs));

		    final JSONObject shapeAttrs = attrs.optJSONObject("shape");
		    if(shapeAttrs!=null) {
		        CTTextBodyProperties bodyPr = shape.getBodyPr();
		        if(bodyPr==null) {
		            bodyPr = Context.getDmlObjectFactory().createCTTextBodyProperties();
		            shape.setBodyPr(bodyPr);
		        }
		        final Object paddingLeft = shapeAttrs.opt("paddingLeft");
		        if(paddingLeft!=null) {
		            if(paddingLeft instanceof Number) {
		                bodyPr.setLIns(Integer.valueOf(((Number)paddingLeft).intValue()*360));
		            }
		            else {
		                bodyPr.setLIns(null);
		            }
		        }
                final Object paddingRight = shapeAttrs.opt("paddingRight");
                if(paddingRight!=null) {
                    if(paddingRight instanceof Number) {
                        bodyPr.setRIns(Integer.valueOf(((Number)paddingRight).intValue()*360));
                    }
                    else {
                        bodyPr.setRIns(null);
                    }
                }
                final Object paddingTop = shapeAttrs.opt("paddingTop");
                if(paddingTop!=null) {
                    if(paddingTop instanceof Number) {
                        bodyPr.setTIns(Integer.valueOf(((Number)paddingTop).intValue()*360));
                    }
                    else {
                        bodyPr.setTIns(null);
                    }
                }
                final Object paddingBottom = shapeAttrs.opt("paddingBottom");
                if(paddingBottom!=null) {
                    if(paddingBottom instanceof Number) {
                        bodyPr.setBIns(Integer.valueOf(((Number)paddingTop).intValue()*360));
                    }
                    else {
                        bodyPr.setBIns(null);
                    }
                }
                final Object spAutoFit = shapeAttrs.opt("autoResizeHeight");
                if(spAutoFit!=null) {
                    if(spAutoFit instanceof Boolean) {
                        bodyPr.setSpAutoFit(((Boolean)spAutoFit).booleanValue()?Context.getDmlObjectFactory().createCTTextShapeAutofit():null);
                    }
                    else {
                        bodyPr.setSpAutoFit(null);
                    }
                }
		    }
		}
		@Override
    	public JSONObject createJSONAttrs(CreateOperationHelper createOperationHelper, JSONObject attrs)
    		throws JSONException, FilterException {

	        DMLHelper.createJsonFromNonVisualDrawingProperties(attrs, shape.getCNvPr());
	        DMLHelper.createJsonFromShapeProperties(attrs, shape.getSpPr(), createOperationHelper.getOperationDocument().getTheme(), shape.getStyle(), true);

            final JSONObject initialShapeAttrs = attrs.optJSONObject("shape");
            final JSONObject shapeAttrs = initialShapeAttrs!=null ? initialShapeAttrs : new JSONObject();
            final CTTextBodyProperties bodyPr = shape.getBodyPr();
	        if(bodyPr!=null) {
	            final Integer paddingLeft = bodyPr.getLIns();
	            if(paddingLeft!=null&&paddingLeft.intValue()!=91440) {
	                shapeAttrs.put("paddingLeft", Commons.coordinateTo100TH_MM(paddingLeft.intValue()));
	            }
	            final Integer paddingRight = bodyPr.getRIns();
	            if(paddingRight!=null&&paddingRight.intValue()!=91440) {
	                shapeAttrs.put("paddingRight", Commons.coordinateTo100TH_MM(paddingRight.intValue()));
	            }
                final Integer paddingTop = bodyPr.getTIns();
                if(paddingTop!=null&&paddingTop.intValue()!=45720) {
                    shapeAttrs.put("paddingTop", Commons.coordinateTo100TH_MM(paddingTop.intValue()));
                }
                final Integer paddingBottom = bodyPr.getBIns();
                if(paddingBottom!=null&&paddingBottom.intValue()!=45720) {
                    shapeAttrs.put("paddingBottom", Commons.coordinateTo100TH_MM(paddingBottom.intValue()));
                }
                final CTTextShapeAutofit spAutoFit = bodyPr.getSpAutoFit();
                if(spAutoFit!=null) {
                    shapeAttrs.put("autoResizeHeight", true);
                }
	        }
            if(initialShapeAttrs==null&&!shapeAttrs.isEmpty()) {
                attrs.put("shape", shapeAttrs);
            }
			return attrs;
    	}
    }

    public static class DrawingML_pic extends DrawingML_base {

    	final private Pic pic;

		protected DrawingML_pic(Pic o, CTWordprocessingGroup parent) {
			super(parent);
			pic = o;
		}
		@Override
		public ShapeType getType() {
			return ShapeType.IMAGE;
		}
		@Override
		public IShape getNext() {
			return getNext(pic);
		}
		@Override
		public void postFixNonConformance(AtomicInteger id) {
			final CTPictureNonVisual picPr = pic.getNvPicPr();
			if(picPr!=null) {
				postFixNonConformance(picPr.getCNvPr(), id);
			}
		}
		@Override
    	public void applyAttrsFromJSON(OperationDocument operationDocument, JSONObject attrs)
    	    throws JSONException, InvalidFormatException, PartUnrecognisedException {

		    pic.setSpPr(DMLHelper.applyShapePropertiesFromJson(pic.getSpPr(), attrs));
            final JSONObject imageAttrs = attrs.optJSONObject("image");
            if(imageAttrs!=null) {
                String imageUrl = Commons.getImageUrlFromImageData(operationDocument.getResourceManager(), imageAttrs.optString("imageData"), "word/media/");
                if(imageUrl==null) {
                    imageUrl = imageAttrs.optString("imageUrl", null);
                }
                if(imageUrl!=null) {
                    final Relationship relationship = Commons.createGraphicRelation(operationDocument, operationDocument.getContextPart(), imageUrl);
                    if(relationship!=null) {
                        CTBlipFillProperties blipFill = pic.getBlipFill();
                        if(blipFill==null) {
                            blipFill = Context.getDmlObjectFactory().createCTBlipFillProperties();
                            pic.setBlipFill(blipFill);
                        }
                        CTBlip blip = blipFill.getBlip();
                        if(blip==null) {
                            blip = Context.getDmlObjectFactory().createCTBlip();
                            blip.setParent(blipFill);
                            blipFill.setBlip(blip);
                        }
                        final boolean linked = relationship.getTargetMode() != null && relationship.getTargetMode().equals("External");
                        if(linked) {
                            blip.setEmbed(null);
                            blip.setLink(relationship.getId());
                        }
                        else {
                            blip.setLink(null);
                            blip.setEmbed(relationship.getId());
                        }
                    }
                }
            }
		}
		@Override
		public JSONObject createJSONAttrs(CreateOperationHelper createOperationHelper, JSONObject attrs)
    		throws JSONException {

			final CTPictureNonVisual pictureNonVisual = pic.getNvPicPr();
			if(pictureNonVisual!=null) {
				DMLHelper.createJsonFromNonVisualDrawingProperties(attrs, pictureNonVisual.getCNvPr());
			}
	        DMLHelper.createJsonFromShapeProperties(attrs, pic.getSpPr(), true);

	        final CTBlipFillProperties blipFillProperties = pic.getBlipFill();
    		if(blipFillProperties!=null) {
    			attrs = DMLHelper.createJsonFromBlipFillProperties(createOperationHelper.getOperationDocument().getContextPart(), attrs, blipFillProperties);
    		}
            return attrs;
    	}
    }

    public static class DrawingML_group extends DrawingML_base {

    	final private CTWordprocessingGroup group;

		protected DrawingML_group(CTWordprocessingGroup o, CTWordprocessingGroup parent) {
			super(parent);
			group = o;
		}
		@Override
		public ShapeType getType() {
			return ShapeType.GROUP;
		}
		@Override
		public void postFixNonConformance(AtomicInteger id) {
			postFixNonConformance(group.getCNvPr(), id);
		}
        @Override
        public IShape getNext() {
            return getNext(group);
        }
        @Override
        public Object getChild() {

            final IndexedNodeList<Object> groupContent = group.getContent();
            if(!groupContent.isEmpty()) {
                final Object child = groupContent.get(0);
                if(child instanceof CTWordprocessingGroup) {
                    return new DrawingML_group((CTWordprocessingGroup)child, group);
                }
                else if (child instanceof CTWordprocessingShape) {
                    return new DrawingML_shape((CTWordprocessingShape)child, group);
                }
                else if (child instanceof Pic) {
                    return new DrawingML_pic((Pic)child, group);
                }
            }
            return null;
        }
        @Override
        public Object insertChild(int number, Component.Type type)
            throws JAXBException {

            final IndexedNodeList<Object> groupContent = group.getContent();
            if(type==Type.AC_GROUP) {
                final CTWordprocessingGroup child = createDrawingMLGroup();
                groupContent.add(number, child);
                return new DrawingML_group(child, group);
            }
            else if(type==Type.AC_SHAPE) {
                final CTWordprocessingShape child = createDrawingMLShape();
                groupContent.add(number, child);
                return new DrawingML_shape(child, group);
            }
            else if(type==Type.AC_IMAGE) {
                final Pic child = createDrawingMLPic();
                groupContent.add(number, child);
                return new DrawingML_pic(child, group);
            }
            return null;
        }
		@Override
    	public void applyAttrsFromJSON(OperationDocument operationDocument, JSONObject attrs)
    	    throws JAXBException, InvalidFormatException, PartUnrecognisedException, JSONException {

		    CTGroupShapeProperties groupProperties = group.getGrpSpPr();
			if(groupProperties==null) {
				groupProperties = Context.getDmlObjectFactory().createCTGroupShapeProperties();
				group.setGrpSpPr(groupProperties);
			}
			CTGroupTransform2D groupTransform2d = groupProperties.getXfrm();
			if(groupTransform2d==null) {
			    groupTransform2d = Context.getDmlObjectFactory().createCTGroupTransform2D();
			    groupProperties.setXfrm(groupTransform2d);
			}
			DMLHelper.applyTransform2DFromJson(groupTransform2d, attrs);
			// now applying the size and pos of the childs
			final CTPoint2D chPos = Context.getDmlObjectFactory().createCTPoint2D();
			chPos.setX(0);
			chPos.setY(0);
			groupTransform2d.setChOff(chPos);
			final CTPositiveSize2D ex = groupTransform2d.getExt();
			if(ex!=null) {
			    final CTPositiveSize2D chEx = Context.getDmlObjectFactory().createCTPositiveSize2D();
			    chEx.setCx(ex.getCx());
			    chEx.setCy(ex.getCy());
			    groupTransform2d.setChExt(chEx);
			}
			DMLHelper.applyFillPropertiesFromJson(groupProperties, attrs);
		}
		@Override
    	public JSONObject createJSONAttrs(CreateOperationHelper createOperationHelper, JSONObject attrs)
    		throws JSONException {

	        DMLHelper.createJsonFromNonVisualDrawingProperties(attrs, group.getCNvPr());
	        final CTGroupShapeProperties groupProperties = group.getGrpSpPr();
			if(groupProperties!=null) {
		        final CTGroupTransform2D groupTransform =groupProperties.getXfrm();
		        if(groupTransform!=null) {
		        	DMLHelper.createJsonFromTransform2D(attrs, groupTransform);
		        }
			}
    		return attrs;
    	}
    }

    public static class DrawingML_root implements IShape {

    	private final Drawing rootObject;

    	// drawing is containing wps and wpg if available, otherwise it is null
    	private DrawingML_base drawing;

    	protected DrawingML_root(Drawing o) {
    		rootObject = o;
			drawing = null;
			final List<GraphicBase> anchorOrInlineList = rootObject.getAnchorOrInline();
            if(!anchorOrInlineList.isEmpty()) {
                final GraphicBase anchorOrInline = anchorOrInlineList.get(0);
                final Graphic graphic = anchorOrInline!=null?anchorOrInline.getGraphic():null;
    			if(graphic!=null) {
                    final GraphicData graphicData = graphic.getGraphicData();
                    if(graphicData!=null) {
                    	final String uri = graphicData.getUri();
                        final List<Object> any = graphicData.getAny();
                    	if(uri!=null&&!any.isEmpty()) {
	                        if(uri.equalsIgnoreCase("http://schemas.microsoft.com/office/word/2010/wordprocessingGroup")) {
	                            Object group = any.get(0);
	                            if(group instanceof JAXBElement && ((JAXBElement<?>)group).getDeclaredType().getName().equals("org.docx4j.dml.wordprocessingGroup2010.CTWordprocessingGroup") ) {
	                            	group = ((JAXBElement<?>)group).getValue();
	                            	any.set(0, group);
	                            }
	                            if(group instanceof CTWordprocessingGroup) {
	                            	drawing = new DrawingML_group((CTWordprocessingGroup)group, null);
	                            }
	                        }
	                        else if(uri.equalsIgnoreCase("http://schemas.microsoft.com/office/word/2010/wordprocessingShape")) {
	                            Object shape = any.get(0);
	                            if(shape instanceof JAXBElement && ((JAXBElement<?>)shape).getDeclaredType().getName().equals("org.docx4j.dml.wordprocessingShape2010.CTWordprocessingShape") ) {
	                            	shape = ((JAXBElement<?>)shape).getValue();
	                            	any.set(0, shape);
	                            }
	                            if(shape instanceof CTWordprocessingShape) {
	                            	drawing = new DrawingML_shape((CTWordprocessingShape)shape, null);
	                            }
	                        }
	                        else if(uri.equalsIgnoreCase("http://schemas.openxmlformats.org/drawingml/2006/picture")) {
	                            Object shape = any.get(0);
	                            if(shape instanceof JAXBElement && ((JAXBElement<?>)shape).getDeclaredType().getName().equals("org.docx4j.dml.picture.Pic") ) {
	                            	shape = ((JAXBElement<?>)shape).getValue();
	                            	any.set(0, shape);
	                            }
	                            if(shape instanceof Pic) {
	                            	drawing = new DrawingML_pic((Pic)shape, null);
	                            }
	                        }
                    	}
                    }
    			}
            }
		}

        final static String namespaces = " xmlns:w=\"http://schemas.openxmlformats.org/wordprocessingml/2006/main\" "
                + "xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\" "
                + "xmlns:wp=\"http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing\"";

        public static Inline createInline(boolean linked, String relId, int cx, int cy)
            throws JAXBException {

            String type = linked ? "r:link" : "r:embed";

            String ml =
                "<wp:inline distT=\"0\" distB=\"0\" distL=\"0\" distR=\"0\"" + namespaces + ">"
                + "<wp:extent cx=\"${cx}\" cy=\"${cy}\"/>"
                + "<wp:effectExtent l=\"0\" t=\"0\" r=\"0\" b=\"0\"/>" + //l=\"19050\"
                "<wp:docPr id=\"${id1}\" name=\"${filenameHint}\" descr=\"${altText}\"/><wp:cNvGraphicFramePr><a:graphicFrameLocks xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\" noChangeAspect=\"1\"/></wp:cNvGraphicFramePr><a:graphic xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\">"
                + "<a:graphicData uri=\"http://schemas.openxmlformats.org/drawingml/2006/picture\">"
                + "<pic:pic xmlns:pic=\"http://schemas.openxmlformats.org/drawingml/2006/picture\"><pic:nvPicPr><pic:cNvPr id=\"${id2}\" name=\"${filenameHint}\"/><pic:cNvPicPr/></pic:nvPicPr><pic:blipFill>"
                + "<a:blip ${rEmbedId}/><a:stretch><a:fillRect/></a:stretch></pic:blipFill>"
                + "<pic:spPr><a:xfrm><a:off x=\"0\" y=\"0\"/><a:ext cx=\"${cx}\" cy=\"${cy}\"/></a:xfrm><a:prstGeom prst=\"rect\"><a:avLst/></a:prstGeom></pic:spPr></pic:pic></a:graphicData></a:graphic>"
                + "</wp:inline>";

            java.util.HashMap<String, String> mappings = new java.util.HashMap<String, String>();

            mappings.put("cx", Integer.toString(cx));
            mappings.put("cy", Integer.toString(cy));
            mappings.put("filenameHint", "");
            mappings.put("altText", "");
            mappings.put("rEmbedId", relId.isEmpty() ? "" : type + "=\"" + relId + "\"");
            mappings.put("id1", "0");           // it seem that this id does not need to be unique (samples/AddImage.java)...
            mappings.put("id2", "1");

            Inline inline = null;
            Object o = org.docx4j.XmlUtils.unmarshallFromTemplate(ml, mappings);
            inline = (Inline) ((JAXBElement<?>) o).getValue();
            return inline;
        }

        public static Anchor createAnchor(boolean linked, String relId, int cx, int cy)
            throws JAXBException {

            String type = linked ? "r:link" : "r:embed";

            String ml =
                "<wp:anchor simplePos=\"0\" distT=\"0\" distB=\"0\" distL=\"0\" distR=\"0\" layoutInCell=\"1\"" + namespaces + ">"
                + "<wp:simplePos x=\"0\" y=\"0\"/>"
                + "<wp:positionH relativeFrom=\"column\"><wp:posOffset>0</wp:posOffset></wp:positionH>"
                + "<wp:positionV relativeFrom=\"paragraph\"><wp:posOffset>0</wp:posOffset></wp:positionV>"
                + "<wp:extent cx=\"${cx}\" cy=\"${cy}\"/>"
                + "<wp:effectExtent l=\"0\" t=\"0\" r=\"0\" b=\"0\"/><wp:wrapTopAndBottom/>"
                + "<wp:docPr id=\"${id1}\" name=\"${filenameHint}\" descr=\"${altText}\"/><wp:cNvGraphicFramePr><a:graphicFrameLocks xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\" noChangeAspect=\"1\"/></wp:cNvGraphicFramePr><a:graphic xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\">"
                + "<a:graphicData uri=\"http://schemas.openxmlformats.org/drawingml/2006/picture\">"
                + "<pic:pic xmlns:pic=\"http://schemas.openxmlformats.org/drawingml/2006/picture\"><pic:nvPicPr><pic:cNvPr id=\"${id2}\" name=\"${filenameHint}\"/><pic:cNvPicPr/></pic:nvPicPr><pic:blipFill>"
                + "<a:blip ${rEmbedId}/><a:stretch><a:fillRect/></a:stretch></pic:blipFill>"
                + "<pic:spPr><a:xfrm><a:off x=\"0\" y=\"0\"/><a:ext cx=\"${cx}\" cy=\"${cy}\"/></a:xfrm><a:prstGeom prst=\"rect\"><a:avLst/></a:prstGeom></pic:spPr></pic:pic></a:graphicData></a:graphic>"
                + "</wp:anchor>";

            java.util.HashMap<String, String> mappings = new java.util.HashMap<String, String>();

            mappings.put("cx", Integer.toString(cx));
            mappings.put("cy", Integer.toString(cy));
            mappings.put("filenameHint", "");
            mappings.put("altText", "");
            mappings.put("rEmbedId", relId.isEmpty() ? "" : type + "=\"" + relId + "\"");
            mappings.put("id1", "0");           // it seem that this id does not need to be unique (samples/AddImage.java)...
            mappings.put("id2", "1");

            Anchor anchor = null;
            Object o = org.docx4j.XmlUtils.unmarshallFromTemplate(ml, mappings);
            anchor = (Anchor) ((JAXBElement<?>) o).getValue();
            return anchor;
        }

        private GraphicBase getAnchorOrInline() {
            final List<GraphicBase> anchorOrInlineContent = rootObject.getAnchorOrInline();
            return anchorOrInlineContent.size() > 0 ? anchorOrInlineContent.get(0) : null;
        }

        private void setAnchorOrInline(GraphicBase o) {
            final List<GraphicBase> anchorOrInlineContent = rootObject.getAnchorOrInline();
            anchorOrInlineContent.clear();
            anchorOrInlineContent.add(o);
        }

        private Inline switchToInlineObject()
            throws JAXBException {

            Anchor anchor = (Anchor)getAnchorOrInline();
            Graphic graphic = anchor.getGraphic();
            Inline inline = createInline(true, "", 0, 0);
            inline.setParent(anchor.getParent());
            inline.setCNvGraphicFramePr(anchor.getCNvGraphicFramePr());
            inline.setDocPr(anchor.getDocPr());
            inline.setGraphic(graphic);
            inline.setExtent(anchor.getExtent());
            inline.setDistB(anchor.getDistB());
            inline.setDistL(anchor.getDistL());
            inline.setDistR(anchor.getDistR());
            inline.setDistT(anchor.getDistT());
            setAnchorOrInline(inline);
            return inline;
        }

        private Anchor switchToAnchorObject()
            throws JAXBException {

            Inline inline = (Inline)getAnchorOrInline();
            Graphic graphic = inline.getGraphic();
            Anchor anchor = createAnchor(true, "", 0, 0);
            anchor.setParent(inline.getParent());
            anchor.setCNvGraphicFramePr(inline.getCNvGraphicFramePr());
            anchor.setDocPr(inline.getDocPr());
            anchor.setGraphic(graphic);
            anchor.setExtent(inline.getExtent());
            anchor.setDistB(inline.getDistB());
            anchor.setDistL(inline.getDistL());
            anchor.setDistR(inline.getDistR());
            anchor.setDistT(inline.getDistT());
            setAnchorOrInline(anchor);
            return anchor;
        }

    	@Override
		public ShapeType getType() {
            ShapeType type = ShapeType.UNDEFINED;
            final List<GraphicBase> anchorOrInlineList = rootObject.getAnchorOrInline();
            if(anchorOrInlineList.isEmpty()) {
            	return ShapeType.UNDEFINED;
            }
            final Object anchorOrInline = anchorOrInlineList.get(0);
            Graphic graphic = null;
			if(anchorOrInline instanceof Anchor)
                graphic = ((Anchor)anchorOrInline).getGraphic();
            else if (anchorOrInline instanceof Inline)
                graphic = ((Inline)anchorOrInline).getGraphic();
			if(graphic==null) {
				return ShapeType.UNDEFINED;
			}
            final GraphicData graphicData = graphic.getGraphicData();
            if(graphicData!=null) {
                final String uri = graphicData.getUri();
                if(uri.equalsIgnoreCase("http://schemas.microsoft.com/office/word/2010/wordprocessingShape"))
                    type = ShapeType.SHAPE;
                else if (uri.equalsIgnoreCase("http://schemas.microsoft.com/office/word/2010/wordprocessingGroup"))
                	type = ShapeType.GROUP;
                else if(uri.equalsIgnoreCase("http://schemas.openxmlformats.org/drawingml/2006/picture"))
                    type = ShapeType.IMAGE;
                else if(uri.equalsIgnoreCase("http://schemas.openxmlformats.org/drawingml/2006/diagram"))
                    type = ShapeType.DIAGRAM;
                else if(uri.equalsIgnoreCase("http://schemas.openxmlformats.org/drawingml/2006/chart"))
                    type = ShapeType.CHART;
            }
            return type;
		}
		@Override
		public void postFixNonConformance(AtomicInteger id) {
			final List<GraphicBase> anchorOrInlineList = rootObject.getAnchorOrInline();
            if(!anchorOrInlineList.isEmpty()) {
            	final GraphicBase graphic = anchorOrInlineList.get(0);
        		DrawingML_base.postFixNonConformance(graphic.getDocPr(), id);
        		if(drawing!=null) {
        		    drawing.postFixNonConformance(id);
        		}
            }
		}
		@Override
    	public void applyAttrsFromJSON(OperationDocument operationDocument, JSONObject attrs)
		    throws JAXBException, JSONException, InvalidFormatException, PartUnrecognisedException {

            if(drawing!=null) {
                drawing.applyAttrsFromJSON(operationDocument, attrs);
            }
            final JSONObject drawingAttrs = attrs.optJSONObject("drawing");
	        if(drawingAttrs!=null) {
	            ObjectFactory dmlObjectFactory = new ObjectFactory();

	            GraphicBase anchor = getAnchorOrInline();
	            Object inlineObj = drawingAttrs.opt("inline");
	            if(inlineObj!=null) {
	                boolean inline = inlineObj instanceof Boolean ? (Boolean)inlineObj : true;
	                if((anchor instanceof Anchor)&&inline)
	                    anchor = switchToInlineObject();
	                else if((anchor instanceof Inline)&&(inline==false))
	                    anchor = switchToAnchorObject();
	            }
	            Iterator<String> keys = drawingAttrs.keys();
	            while(keys.hasNext()) {
	                String attr = keys.next();
	                Object value = drawingAttrs.get(attr);
	                if(attr.equals("width")) {
	                    if(value instanceof Integer) {
	                        CTPositiveSize2D extend = anchor.getExtent();
	                        if(extend==null) {
	                            extend = Context.getDmlObjectFactory().createCTPositiveSize2D();
	                            anchor.setExtent(extend);
	                        }
	                        extend.setCx(((Integer)value).intValue()*360);
	                    }
	                }
	                else if(attr.equals("height")) {
	                    if(value instanceof Integer) {
	                        CTPositiveSize2D extend = anchor.getExtent();
	                        if(extend==null) {
	                            extend = Context.getDmlObjectFactory().createCTPositiveSize2D();
	                            anchor.setExtent(extend);
	                        }
	                        extend.setCy(((Integer) value).intValue()*360);
	                    }
	                }
	                else if(attr.equals("marginLeft")) {
	                    if(value instanceof Integer) {
	                        anchor.setDistL(((Integer)value).longValue()*360);
	                    }
	                }
	                else if(attr.equals("marginTop")) {
	                    if(value instanceof Integer) {
	                        anchor.setDistT(((Integer)value).longValue()*360);
	                    }
	                }
	                else if(attr.equals("marginRight")) {
	                    if(value instanceof Integer) {
	                        anchor.setDistR(((Integer)value).longValue()*360);
	                    }
	                }
	                else if(attr.equals("marginBottom")) {
	                    if(value instanceof Integer) {
	                        anchor.setDistB(((Integer)value).longValue()*360);
	                    }
	                }
	                // !!!!.... only attributes for anchored drawings are following now !!!!
	                else if(anchor instanceof Anchor) {
	                    if(attr.equals("anchorHorBase")) {
	                        if(value instanceof String) {
	                            CTPosH ctPosH = ((Anchor) anchor).getPositionH();
	                            if(ctPosH==null)
	                                ctPosH = dmlObjectFactory.createCTPosH();
	                            ctPosH.setRelativeFrom(STRelFromH.fromValue((String)value));
	                        }
	                    }
	                    else if(attr.equals("anchorHorAlign")) {
	                        if(value instanceof String) {
	                            CTPosH ctPosH = ((Anchor) anchor).getPositionH();
	                            if(ctPosH==null)
	                                ctPosH = dmlObjectFactory.createCTPosH();
	                            if(((String)value).equals("offset"))
	                                ctPosH.setAlign(null);
	                            else
	                                ctPosH.setAlign(STAlignH.fromValue((String)value));
	                        }
	                    }
	                    else if(attr.equals("anchorHorOffset")) {
	                        if(value instanceof Integer) {
	                            CTPosH ctPosH = ((Anchor) anchor).getPositionH();
	                            if(ctPosH==null)
	                                ctPosH = dmlObjectFactory.createCTPosH();
	                            ctPosH.setPosOffset((Integer)value * 360);
	                        }
	                    }
	                    else if(attr.equals("anchorVertBase")) {
	                        if(value instanceof String) {
	                            CTPosV ctPosV = ((Anchor) anchor).getPositionV();
	                            if(ctPosV==null)
	                                ctPosV = dmlObjectFactory.createCTPosV();
	                            ctPosV.setRelativeFrom(STRelFromV.fromValue((String)value));
	                        }
	                    }
	                    else if(attr.equals("anchorVertAlign")) {
	                        if(value instanceof String) {
	                            CTPosV ctPosV = ((Anchor) anchor).getPositionV();
	                            if(ctPosV==null)
	                                ctPosV = dmlObjectFactory.createCTPosV();
	                            if(((String)value).equals("offset"))
	                                ctPosV.setAlign(null);
	                            else
	                                ctPosV.setAlign(STAlignV.fromValue((String)value));
	                        }
	                    }
	                    else if(attr.equals("anchorVertOffset")) {
	                        if(value instanceof Integer) {
	                            CTPosV ctPosV = ((Anchor) anchor).getPositionV();
	                            if(ctPosV==null)
	                                ctPosV = dmlObjectFactory.createCTPosV();
	                            ctPosV.setPosOffset((Integer)value * 360);
	                        }
	                    }
	                    else if(attr.equals("textWrapMode")) {
	                        if(value instanceof String) {
	                            CTWrapNone      ctWrapNone = null;
	                            CTWrapSquare    ctWrapSquare = null;
	                            CTWrapTight     ctWrapTight = null;
	                            CTWrapThrough   ctWrapThrough = null;
	                            CTWrapTopBottom ctWrapTopBottom = null;
	                            if(((String)value).equals("none")) {
	                                ctWrapNone = ((Anchor)anchor).getWrapNone();
	                                if(ctWrapNone==null)
	                                    ctWrapNone = dmlObjectFactory.createCTWrapNone();
	                            }
	                            else if(((String)value).equals("square")) {
	                                ctWrapSquare = ((Anchor)anchor).getWrapSquare();
	                                if(ctWrapSquare==null) {
	                                    ctWrapSquare = dmlObjectFactory.createCTWrapSquare();
	                                    ctWrapSquare.setWrapText(STWrapText.BOTH_SIDES);
	                                }
	                            }
	                            else if(((String)value).equals("tight")) {
	                                ctWrapTight = ((Anchor)anchor).getWrapTight();
	                                if(ctWrapTight==null) {
	                                    ctWrapTight = dmlObjectFactory.createCTWrapTight();
	                                    ctWrapTight.setWrapText(STWrapText.BOTH_SIDES);
	                                }
	                            }
	                            else if(((String)value).equals("through")) {
	                                ctWrapThrough = ((Anchor)anchor).getWrapThrough();
	                                if(ctWrapThrough==null) {
	                                    ctWrapThrough = dmlObjectFactory.createCTWrapThrough();
	                                    ctWrapThrough.setWrapText(STWrapText.BOTH_SIDES);
	                                }
	                            }
	                            else if(((String)value).equals("topAndBottom")) {
	                                ctWrapTopBottom = ((Anchor)anchor).getWrapTopAndBottom();
	                                if(ctWrapTopBottom==null)
	                                    ctWrapTopBottom = dmlObjectFactory.createCTWrapTopBottom();
	                            }
	                            ((Anchor)anchor).setWrapNone(ctWrapNone);
	                            ((Anchor)anchor).setWrapSquare(ctWrapSquare);
	                            ((Anchor)anchor).setWrapTight(ctWrapTight);
	                            ((Anchor)anchor).setWrapThrough(ctWrapThrough);
	                            ((Anchor)anchor).setWrapTopAndBottom(ctWrapTopBottom);
	                        }
	                    }
	                }
	            }

	            // either "align" or "position" is to be used, otherwise Word won't open the document
	            if(anchor instanceof Anchor) {
	                CTPosH ctPosH = ((Anchor) anchor).getPositionH();
	                if(ctPosH!=null) {
	                    if(ctPosH.getAlign()!=null)
	                        ctPosH.setPosOffset(null);
	                    else {
	                        Integer pos = ctPosH.getPosOffset();
	                        if(pos==null)
	                            ctPosH.setPosOffset(0);
	                    }
	                }
	                CTPosV ctPosV = ((Anchor) anchor).getPositionV();
	                if(ctPosV!=null) {
	                    if(ctPosV.getAlign()!=null)
	                        ctPosV.setPosOffset(null);
	                    else {
	                        Integer pos = ctPosV.getPosOffset();
	                        if(pos==null)
	                            ctPosV.setPosOffset(0);
	                    }
	                }
	                // textwrapside can't be set inside the above while loop, because it depends to the
	                // wrapmode that is used. late applying preserves us from taking care of the property order.
	                if (drawingAttrs.has("textWrapSide")) {
	                    Object value = drawingAttrs.get("textWrapSide");
	                    if(value instanceof String) {
	                        String wrapText = ((String)value).equals("both") ? "bothSides" : (String)value;
	                        if(((Anchor)anchor).getWrapSquare()!=null)
	                            ((Anchor)anchor).getWrapSquare().setWrapText(STWrapText.fromValue(wrapText));
	                        else if (((Anchor)anchor).getWrapTight()!=null)
	                            ((Anchor)anchor).getWrapTight().setWrapText(STWrapText.fromValue(wrapText));
	                        else if (((Anchor)anchor).getWrapThrough()!=null)
	                            ((Anchor)anchor).getWrapThrough().setWrapText(STWrapText.fromValue(wrapText));
	                    }
	                }
	            }
	        }
    	}

	    final static String[] enumRelFromH = { "margin", "page", "column", "character", "leftMargin", "rightMargin", "insideMargin", "outsideMargin" };
	    final static String[] enumSTAlignH = { "left", "right", "center", "inside", "outside" };
	    final static String[] enumRelFromV = { "margin", "page", "paragraph", "line", "topmargin", "bottomMargin", "insideMargin", "outsideMargin" };
	    final static String[] enumSTAlignV = { "top", "bottom", "center", "inside", "outside" };
	    final static String[] enumSTWrapText = { "both", "left", "right", "largest" };

		@Override
    	public JSONObject createJSONAttrs(CreateOperationHelper createOperationHelper, JSONObject attrs)
    		throws JSONException, ParseException, FilterException {

            if(drawing!=null) {
                attrs = drawing.createJSONAttrs(createOperationHelper, attrs);
            }
	        final GraphicBase anchor = getAnchorOrInline();

	        DMLHelper.createJsonFromNonVisualDrawingProperties(attrs, anchor.getDocPr());

	        final JSONObject initialDrawingAttrs = attrs.optJSONObject("drawing");
	        final JSONObject drawingAttrs = initialDrawingAttrs!=null ? initialDrawingAttrs : new JSONObject();

	        DMLHelper.createJsonFromPositiveSize2D(drawingAttrs, anchor.getExtent());

	        if(anchor!=null) {
	            drawingAttrs.put("inline", anchor instanceof Inline);

	            // applying margin properties
	            final Long DistL = anchor.getDistL();
	            final Long DistT = anchor.getDistT();
	            final Long DistR = anchor.getDistR();
	            final Long DistB = anchor.getDistB();
	            if((DistL!=null) && (DistL.intValue() != 0))
	                drawingAttrs.put("marginLeft", DistL.intValue() / 360);
	            if((DistT!=null) && (DistT.intValue() != 0))
	                drawingAttrs.put("marginTop", DistT.intValue() / 360);
	            if((DistR!=null) && (DistR.intValue() != 0))
	                drawingAttrs.put("marginRight", DistR.intValue() / 360);
	            if((DistB!=null) && (DistB.intValue() != 0))
	                drawingAttrs.put("marginBottom", DistB.intValue() / 360);

	            // positioning in word
	            if(anchor instanceof Anchor) {
	                // horizontal
	                CTPosH ctPosH = ((Anchor) anchor).getPositionH();
	                if(ctPosH!=null) {
	                    STRelFromH stRelFrom = ctPosH.getRelativeFrom();
	                    if(stRelFrom!=null) {
	                        int val = stRelFrom.ordinal();
	                        if(val>=enumRelFromH.length)
	                            val = 0;
	                        drawingAttrs.put("anchorHorBase", enumRelFromH[val] );
	                    }
	                    STAlignH stAlignH = ctPosH.getAlign();
	                    if(stAlignH!=null) {
	                        int val = stAlignH.ordinal();
	                        if(val>=enumSTAlignH.length)
	                            val = 0;
	                        drawingAttrs.put("anchorHorAlign", enumSTAlignH[val]);
	                    }
	                    else
	                        drawingAttrs.put("anchorHorAlign", "offset");
	                    Integer val = ctPosH.getPosOffset();
	                    if(val!=null) {
	                        drawingAttrs.put("anchorHorOffset", val.intValue() / 360);
	                    }
	                }
	                // vertical
	                CTPosV ctPosV = ((Anchor) anchor).getPositionV();
	                if(ctPosV!=null) {
	                    STRelFromV stRelFrom = ctPosV.getRelativeFrom();
	                    if(stRelFrom!=null) {
	                        int val = stRelFrom.ordinal();
	                        if(val>=enumRelFromV.length)
	                            val = 0;
	                        drawingAttrs.put("anchorVertBase", enumRelFromV[val] );
	                    }
	                    STAlignV stAlignV = ctPosV.getAlign();
	                    if(stAlignV!=null) {
	                        int val = stAlignV.ordinal();
	                        if(val>=enumSTAlignV.length)
	                            val = 0;
	                        drawingAttrs.put("anchorVertAlign", enumSTAlignV[val]);
	                    }
	                    else
	                        drawingAttrs.put("anchorVertAlign", "offset");
	                    Integer val = ctPosV.getPosOffset();
	                    if(val!=null) {
	                        drawingAttrs.put("anchorVertOffset",val.intValue() / 360);
	                    }
	                }
	                // textwrapmode
	                String textWrapMode = null;
	                STWrapText textWrapSide = null;
	                CTWrapNone wrapNone = ((Anchor)anchor).getWrapNone();
	                if(wrapNone!=null)
	                    textWrapMode = "none";
	                else {
	                    CTWrapSquare wrapSquare = ((Anchor)anchor).getWrapSquare();
	                    if(wrapSquare!=null) {
	                        textWrapMode = "square";
	                        textWrapSide = wrapSquare.getWrapText();
	                    }
	                    else {
	                        CTWrapTight wrapTight = ((Anchor)anchor).getWrapTight();
	                        if(wrapTight!=null) {
	                            textWrapMode = "tight";
	                            textWrapSide = wrapTight.getWrapText();
	                        }
	                        else {
	                            CTWrapThrough wrapThrough = ((Anchor)anchor).getWrapThrough();
	                            if(wrapThrough!=null) {
	                                textWrapMode = "through";
	                                textWrapSide = wrapThrough.getWrapText();
	                            }
	                            else {
	                                CTWrapTopBottom wrapTopAndBottom = ((Anchor)anchor).getWrapTopAndBottom();
	                                if(wrapTopAndBottom!=null)
	                                    textWrapMode = "topAndBottom";
	                            }
	                        }
	                    }
	                }
	                if(textWrapMode!=null)
	                    drawingAttrs.put("textWrapMode", textWrapMode);
	                if(textWrapSide!=null) {
	                    int val = textWrapSide.ordinal();
	                    if(val>=enumSTWrapText.length)
	                        val = 0;
	                    drawingAttrs.put("textWrapSide", enumSTWrapText[val]);
	                }
	            }
	        }
	        if(initialDrawingAttrs==null&&!drawingAttrs.isEmpty()) {
	            attrs.put("drawing", drawingAttrs);
	        }
    		return attrs;
    	}
		@Override
		public IShape getNext() {

		    return null;
		}
		@Override
		public Object getChild() {

		    return drawing!=null?drawing.getChild():null;
		}
		@Override
		public Object insertChild(int number, Component.Type type)
		    throws JAXBException {

		    return drawing==null?null:drawing.insertChild(number, type);
		}
    }
}
